Package: fastml
Type: Package
Title: Guarded Resampling Workflows for Safe and Automated Machine
        Learning in R
Version: 0.7.9
Authors@R: c(person("Selcuk", "Korkmaz", email="selcukorkmaz@gmail.com", role=c("aut", "cre"),
                    comment = c(ORCID = "0000-0003-4632-6850")), 
             person("Dincer", "Goksuluk", email="dincer.goksuluk@gmail.com", role="aut",
                    comment = c(ORCID = "0000-0002-2752-7668")), 
             person("Eda", "Karaismailoglu", email="eda.karaismailoglu@sbu.edu.tr", role="aut",
                    comment = c(ORCID = "0000-0003-3085-7809")))
Description: Provides a guarded resampling workflow for training and evaluating machine-learning models. 
	When the guarded resampling path is used, preprocessing and model fitting are re-estimated within 
	each resampling split to reduce leakage risk. Supports multiple resampling schemes, integrates 
	with established engines in the 'tidymodels' ecosystem, and aims to improve evaluation reliability by 
	coordinating preprocessing, fitting, and evaluation within supported workflows. Offers a lightweight 
	AutoML-style workflow by automating model training, resampling, and tuning across multiple algorithms, 
	while keeping evaluation design explicit and user-controlled.
Encoding: UTF-8
License: MIT + file LICENSE
URL: https://selcukorkmaz.github.io/fastml-tutorial/,
        https://github.com/selcukorkmaz/fastml
BugReports: https://github.com/selcukorkmaz/fastml/issues
Depends: R (>= 4.1.0)
Imports: stats, recipes, dplyr, ggplot2, reshape2, rsample, parsnip,
        tune, workflows, yardstick, tibble, rlang, dials, RColorBrewer,
        doFuture, foreach, finetune, future, viridisLite, magrittr,
        pROC, janitor, stringr, broom, tidyr, purrr, survival, xgboost
Suggests: testthat (>= 3.0.0), withr, baguette, C50, DALEX, discrim,
        fairmodels, flexsurv, iBreakDown, iml, lime, modelStudio, pdp,
        plsmod, probably, ranger, rstpm2, survRM2, aorsf, censored,
        crayon, kernlab, klaR, kknn, keras, lightgbm, rstanarm,
        patchwork, GGally, glmnet, themis, DT, UpSetR, VIM, dbscan,
        gridExtra, htmlwidgets, kableExtra, moments, naniar, plotly,
        scales, skimr, sparsediscrim, knitr, rmarkdown, pec
RoxygenNote: 7.3.3
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-08-19 11:34:52 UTC; selcukkorkmaz
Author: Selcuk Korkmaz [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4632-6850>),
  Dincer Goksuluk [aut] (ORCID: <https://orcid.org/0000-0002-2752-7668>),
  Eda Karaismailoglu [aut] (ORCID:
    <https://orcid.org/0000-0003-3085-7809>)
Maintainer: Selcuk Korkmaz <selcukorkmaz@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-19 12:50:02 UTC
