# Generated by roxygen2: do not edit by hand

S3method(plot,fastml)
S3method(plot,fastml_stability)
S3method(predict,fastml)
S3method(predict_risk,default)
S3method(predict_risk,fastml_native_survival)
S3method(predict_risk,workflow)
S3method(predict_survival,default)
S3method(predict_survival,fastml_native_survival)
S3method(predict_survival,workflow)
S3method(print,fastml_stability)
S3method(summary,fastml)
export(availableMethods)
export(compare_defaults)
export(counterfactual_explain)
export(estimate_tuning_time)
export(explain_ale)
export(explain_dalex)
export(explain_lime)
export(explain_stability)
export(fastexplain)
export(fastexplore)
export(fastml)
export(fastml_compute_holdout_results)
export(flatten_and_rename_models)
export(get_best_model_idx)
export(get_best_model_names)
export(get_best_workflows)
export(get_default_differences)
export(get_default_engine)
export(get_default_params)
export(get_default_params_with_warnings)
export(get_default_tune_params)
export(get_engine_names)
export(get_expanded_tune_params)
export(get_model_engine_names)
export(get_parsnip_default_engine)
export(get_parsnip_default_params)
export(get_tuning_complexity)
export(get_tuning_params_for_complexity)
export(interaction_strength)
export(load_model)
export(plot_ice)
export(predict_model.model_fit)
export(predict_risk)
export(predict_survival)
export(print_default_differences)
export(print_tuning_presets)
export(process_model)
export(recommend_tuning_config)
export(reset_default_warnings)
export(sanitize)
export(save.fastml)
export(save_fastml)
export(surrogate_tree)
export(train_models)
export(validate_defaults_registry)
export(warn_default_override)
importFrom(RColorBrewer,brewer.pal)
importFrom(broom,glance)
importFrom(broom,tidy)
importFrom(dials,finalize)
importFrom(dials,grid_latin_hypercube)
importFrom(dials,grid_regular)
importFrom(dials,range_set)
importFrom(dials,value_set)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,n_distinct)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,rename_with)
importFrom(dplyr,row_number)
importFrom(dplyr,select)
importFrom(dplyr,starts_with)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(dplyr,where)
importFrom(finetune,control_race)
importFrom(finetune,tune_race_anova)
importFrom(ggplot2,aes)
importFrom(ggplot2,autoplot)
importFrom(ggplot2,coord_equal)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_errorbarh)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_path)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_rug)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,position_dodge)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,ylim)
importFrom(graphics,plot)
importFrom(janitor,make_clean_names)
importFrom(magrittr,"%>%")
importFrom(pROC,auc)
importFrom(pROC,multiclass.roc)
importFrom(pROC,roc)
importFrom(parsnip,bag_tree)
importFrom(parsnip,boost_tree)
importFrom(parsnip,decision_tree)
importFrom(parsnip,extract_parameter_set_dials)
importFrom(parsnip,fit)
importFrom(parsnip,linear_reg)
importFrom(parsnip,logistic_reg)
importFrom(parsnip,mlp)
importFrom(parsnip,multinom_reg)
importFrom(parsnip,naive_Bayes)
importFrom(parsnip,nearest_neighbor)
importFrom(parsnip,pls)
importFrom(parsnip,predict.model_fit)
importFrom(parsnip,proportional_hazards)
importFrom(parsnip,rand_forest)
importFrom(parsnip,set_engine)
importFrom(parsnip,set_mode)
importFrom(parsnip,survival_reg)
importFrom(parsnip,svm_linear)
importFrom(parsnip,svm_rbf)
importFrom(purrr,flatten)
importFrom(recipes,all_nominal_predictors)
importFrom(recipes,all_numeric_predictors)
importFrom(recipes,all_outcomes)
importFrom(recipes,all_predictors)
importFrom(recipes,bake)
importFrom(recipes,prep)
importFrom(recipes,recipe)
importFrom(recipes,step_center)
importFrom(recipes,step_dummy)
importFrom(recipes,step_impute_bag)
importFrom(recipes,step_impute_knn)
importFrom(recipes,step_impute_median)
importFrom(recipes,step_naomit)
importFrom(recipes,step_novel)
importFrom(recipes,step_rm)
importFrom(recipes,step_scale)
importFrom(recipes,step_unknown)
importFrom(recipes,step_zv)
importFrom(reshape2,dcast)
importFrom(reshape2,melt)
importFrom(rlang,.data)
importFrom(rlang,as_string)
importFrom(rlang,call2)
importFrom(rlang,call_modify)
importFrom(rlang,expr)
importFrom(rlang,get_env)
importFrom(rlang,get_expr)
importFrom(rlang,sym)
importFrom(rlang,syms)
importFrom(rsample,analysis)
importFrom(rsample,assessment)
importFrom(rsample,bootstraps)
importFrom(rsample,group_vfold_cv)
importFrom(rsample,initial_split)
importFrom(rsample,nested_cv)
importFrom(rsample,rolling_origin)
importFrom(rsample,testing)
importFrom(rsample,training)
importFrom(rsample,validation_split)
importFrom(rsample,vfold_cv)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,aggregate)
importFrom(stats,approx)
importFrom(stats,approxfun)
importFrom(stats,as.formula)
importFrom(stats,ave)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,delete.response)
importFrom(stats,formula)
importFrom(stats,integrate)
importFrom(stats,logLik)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.offset)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,nobs)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stringr,str_detect)
importFrom(survival,Surv)
importFrom(survival,basehaz)
importFrom(survival,coxph)
importFrom(survival,psurvreg)
importFrom(survival,strata)
importFrom(survival,survfit)
importFrom(survival,untangle.specials)
importFrom(tibble,as_tibble)
importFrom(tibble,is_tibble)
importFrom(tibble,tibble)
importFrom(tidyr,pivot_wider)
importFrom(tune,control_bayes)
importFrom(tune,control_grid)
importFrom(tune,extract_fit_parsnip)
importFrom(tune,finalize_model)
importFrom(tune,finalize_workflow)
importFrom(tune,select_best)
importFrom(tune,tune)
importFrom(tune,tune_bayes)
importFrom(tune,tune_grid)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(viridisLite,viridis)
importFrom(workflows,add_model)
importFrom(workflows,add_recipe)
importFrom(workflows,extract_fit_parsnip)
importFrom(workflows,extract_recipe)
importFrom(workflows,pull_workflow_preprocessor)
importFrom(workflows,pull_workflow_spec)
importFrom(workflows,workflow)
importFrom(yardstick,accuracy)
importFrom(yardstick,conf_mat)
importFrom(yardstick,f_meas)
importFrom(yardstick,kap)
importFrom(yardstick,mae)
importFrom(yardstick,metric_set)
importFrom(yardstick,new_class_metric)
importFrom(yardstick,precision)
importFrom(yardstick,rmse)
importFrom(yardstick,roc_auc)
importFrom(yardstick,rsq)
importFrom(yardstick,sens)
importFrom(yardstick,spec)
