% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defaults_registry.R
\name{warn_default_override}
\alias{warn_default_override}
\title{Warn About Default Overrides}
\usage{
warn_default_override(
  algo,
  task,
  fastml_engine,
  fastml_params = NULL,
  verbose = FALSE,
  warn_once = TRUE
)
}
\arguments{
\item{algo}{Algorithm name.}

\item{task}{Task type.}

\item{fastml_engine}{fastml's default engine for this algorithm.}

\item{fastml_params}{fastml's default parameters (optional).}

\item{verbose}{If TRUE, always show the message (as a message, not warning).}

\item{warn_once}{If TRUE (default), only warn once per algorithm per session.}
}
\value{
Invisibly returns the comparison result.
}
\description{
Issues a warning if fastml defaults differ from parsnip defaults and
the warning hasn't been shown yet in this session.
}
