% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xgboost_survival.R
\name{predict_survival}
\alias{predict_survival}
\alias{predict_survival.fastml_native_survival}
\alias{predict_survival.workflow}
\alias{predict_survival.default}
\title{Predict survival probabilities from a survival model}
\usage{
predict_survival(fit, newdata, times, ...)

\method{predict_survival}{fastml_native_survival}(fit, newdata, times, ...)

\method{predict_survival}{workflow}(fit, newdata, times, ...)

\method{predict_survival}{default}(fit, newdata, times, ...)
}
\arguments{
\item{fit}{A fitted survival model.}

\item{newdata}{A data frame of predictors for which to compute survival curves.}

\item{times}{Numeric vector of evaluation times.}

\item{...}{Additional arguments passed to methods.}
}
\value{
A numeric matrix with one row per observation and one column per time.
}
\description{
Predict survival probabilities from a survival model
}
