% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defaults_registry.R
\docType{data}
\name{.fastml_warned_defaults}
\alias{.fastml_warned_defaults}
\title{Environment for Tracking Warned Defaults}
\format{
An object of class \code{environment} of length 0.
}
\usage{
.fastml_warned_defaults
}
\description{
Internal environment to track which default warnings have been shown
in the current session to avoid duplicate warnings.
}
\keyword{internal}
