% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/explain_stability.R
\name{print.fastml_stability}
\alias{print.fastml_stability}
\title{Print method for fastml_stability objects}
\usage{
\method{print}{fastml_stability}(x, top_n = 10, ...)
}
\arguments{
\item{x}{A fastml_stability object from explain_stability().}

\item{top_n}{Integer. Number of top features to display. Default is 10.}

\item{...}{Additional arguments (ignored).}
}
\value{
The input object invisibly.
}
\description{
Print method for fastml_stability objects
}
