% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_model_helpers.R
\name{map_brier_values}
\alias{map_brier_values}
\title{Map Brier Curve Values to Specific Horizons}
\usage{
map_brier_values(curve, eval_times, horizons)
}
\arguments{
\item{curve}{Numeric vector of Brier scores from \code{compute_ibrier}.}

\item{eval_times}{Numeric vector of times corresponding to \code{curve}.}

\item{horizons}{Numeric vector of target time horizons to extract.}
}
\value{
A numeric vector of Brier scores corresponding to \code{horizons}.
}
\description{
Extracts Brier score values from a pre-computed curve at specific
time horizons by finding the closest matching evaluation time.
}
\keyword{internal}
