% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_model_helpers.R
\name{assign_risk_group}
\alias{assign_risk_group}
\title{Assign Risk Groups}
\usage{
assign_risk_group(risk_vec)
}
\arguments{
\item{risk_vec}{Numeric vector of predicted risk scores.}
}
\value{
A character vector of "low", "high", or \code{NA}.
}
\description{
Dichotomizes a continuous risk vector into "low" and "high" risk groups
based on the median.
}
\keyword{internal}
