% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defaults_registry.R
\name{defaults_registry}
\alias{defaults_registry}
\title{Defaults Registry for Engine and Parameter Transparency}
\description{
Functions to track, compare, and warn about differences between
  fastml defaults and parsnip defaults, providing users with full transparency
  and control over model configuration.
}
