% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defaults_registry.R
\name{get_default_differences}
\alias{get_default_differences}
\title{Get All Default Differences Summary}
\usage{
get_default_differences(algorithms, task = "classification")
}
\arguments{
\item{algorithms}{Character vector of algorithm names.}

\item{task}{Task type ("classification", "regression", or "survival").}
}
\value{
A data frame summarizing the differences.
}
\description{
Returns a summary of all differences between fastml and parsnip defaults
for the specified algorithms.
}
