% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tuning_config.R
\name{get_expanded_tune_params}
\alias{get_expanded_tune_params}
\title{Expanded Default Tuning Parameters}
\usage{
get_expanded_tune_params(algo, train_data, label, engine)
}
\arguments{
\item{algo}{Algorithm name.}

\item{train_data}{Training data frame.}

\item{label}{Outcome variable name.}

\item{engine}{Engine name.}
}
\value{
A list of expanded tuning parameter ranges.
}
\description{
Returns expanded tuning parameter ranges that provide better coverage
than the minimal defaults. These are used when \code{tuning_complexity}
is set to "thorough" or "exhaustive".
}
\keyword{internal}
